Package: optimx
Version: 2024-12.2
Date: 2024-12-02
Title: Expanded Replacement and Extension of the 'optim' Function
Authors@R: c( person(given = c("John", "C"), family = "Nash", role =
    c("aut", "cre"), email = "profjcnash@gmail.com"), person(given =
    "Ravi", family = "Varadhan", role = "aut", email =
    "RVaradhan@jhmi.edu"), person(given = "Hans W", family =
    "Borchers", role = "ctb", email =  "hwborchers@gmail.com"),  
    person(given = "Gabor", family = "Grothendieck", role = "ctb", email =
    "ggrothendieck@gmail.com") )
Maintainer: John C Nash <profjcnash@gmail.com>
Description: Provides a replacement and extension of the optim()
    function to call to several function minimization codes in R in a single
    statement. These methods handle smooth, possibly box constrained functions 
    of several or many parameters. Note that function 'optimr()' was prepared to
    simplify the incorporation of minimization codes going forward. Also implements some
    utility codes and some extra solvers, including safeguarded Newton methods.
    Many methods previously separate are now included here.
    This is the version for CRAN.
License: GPL-2
URL: https://github.com/nashjc/optimx
BugReports: https://github.com/nashjc/optimx/issues
LazyLoad: Yes
Imports: numDeriv, nloptr, pracma
NeedsCompilation: no
Suggests: knitr, rmarkdown, setRNG, BB, ucminf, minqa, dfoptim,
        lbfgsb3c, lbfgs, subplex, marqLevAlg, testthat (>= 3.0.0),
        R.rsp
VignetteBuilder: R.rsp
Config/testthat/edition: 3
Config/build/clean-inst-doc: false
Packaged: 2024-12-10 02:16:41 UTC; john
Author: John C Nash [aut, cre],
  Ravi Varadhan [aut],
  Hans W Borchers [ctb],
  Gabor Grothendieck [ctb]
Repository: CRAN
Date/Publication: 2024-12-10 10:30:01 UTC
