importFrom("stats", "nlm", "nlminb", "optim")
importFrom("numDeriv", "grad", "hessian", "jacobian")
importFrom("utils", "str")
importFrom("nloptr", "nloptr")
# importFrom("nloptr", "tnewton")
# importFrom("nloptr", "slsqp")
importFrom("pracma", "anms")
importFrom("pracma", "nelder_mead")
importFrom("utils", "getFromNamespace")
importFrom("utils", "install.packages")
# importFrom("marqLevAlg", "mla")
export(
  "axsearch",
  "bmchk",
  "bmstep",
  "checksolver",
  "checkallsolvers",
  "ctrldefault",
  "dispdefault",
  "coef<-",
  "fnchk",
  "gHgen",
  "gHgenb",
  "grback",
  "grcentral",
  "grchk",
  "grfwd",
  "grnd",
  "grpracma",
  "hesschk", 
  "hjn",
  "kktchk",
  "multistart",
  "ncg",
  "nvm",
  "opm",
  "opm2optimr",
  "optchk",
  "optimr",
  "optimr2opm",
  "optimx",
  "optsp",
  "pd_check",
  "polyopt",
  "proptimr",
  "Rcgmin",
  "Rcgminu",
  "Rcgminb",
  "tn",
  "tnbc",
  "Rvmmin",
  "Rvmminb",
  "Rvmminu",
  "scalechk",
  "snewtm",
  "snewton"
#  "snewtonm"
)
S3method("[", "opm")
S3method("as.data.frame", "opm")
S3method("coef", "opm")
S3method("coef<-", "opm")
S3method("summary", "opm")
S3method("[", "optimx")
S3method("as.data.frame", "optimx")
S3method("coef", "optimx")
S3method("coef<-", "optimx")
S3method("summary", "optimx")

